/**
 * This jQuery plugin displays map and its components using MapLibre.
 * @version 1.0
 */
(function($, window, document, undefined) {
    "use strict";

    function MapLibreMaps(element, map_data) {
        var options;
        this.element = element;
        map_data = JSON.parse(atob(map_data));
        this.map_data = $.extend({}, {}, map_data);
        options = this.map_data.map_options;
        Object.keys(options).forEach((k) => options[k] == null && delete options[k]);
        this.settings = $.extend(true, {
            "zoom": 3,
            "map_type_id": "streets",
            "scroll_wheel": true,
            "map_visual_refresh": false,
            "full_screen_control": false,
            "full_screen_control_position": "bottom-right",
            "zoom_control": true,
            "zoom_control_style": "small",
            "zoom_control_position": "top-left",
            "map_type_control": true,
            "map_type_control_style": "horizontal-bar",
            "map_type_control_position": "top-right",
            "scale_control": true,
            "overview_map_control": true,
            "center_lat": "16.07526",
            "center_lng": "108.224212",
            "draggable": true
        }, options);
        console.log('options', options);
        this.container = $("div[rel='" + $(this.element).attr("id") + "']");
        $(document).find(".trackasia_map_container_placeholder").remove();
        $(document).find(".trackasia_hide_map_container").removeClass("trackasia_hide_map_container");

        this.geocoder = {
            geocode: function(address, callback) {
                var url = `https://maps.track-asia.com/api/v2/geocode/json?address=${encodeURIComponent(address)}&key=${trackasiagl.accessToken}`;
                $.getJSON(url, function(data) {
                    if (data.results && data.results.length > 0) {
                        callback(data.results[0]);
                    } else {
                        callback(null);
                    }
                });
            },

            placeIdGeocode: function(placeId, callback) {
                var url = `https://maps.track-asia.com/api/v2/geocode/json?place_id=${encodeURIComponent(placeId)}&key=${trackasiagl.accessToken}`;
                $.getJSON(url, function(data) {
                    if (data.results && data.results.length > 0 && callback) {
                        callback(data.results[0]);
                    } else {
                        callback(null);
                    }
                });
            },
        
            reverseGeocode: function(latlng, callback) {
                var url = `https://maps.track-asia.com/api/v2/geocode/json?latlng=${latlng}&key=${trackasiagl.accessToken}`;
                $.getJSON(url, function(data) {
                    if (data.results && data.results.length > 0) {
                        callback(data.results[0]);
                    } else {
                        callback(null);
                    }
                });
            },
        
            autocomplete: function(input, bounds, size, callback) {
                var url = `https://maps.track-asia.com/api/v2/place/autocomplete/json?input=${encodeURIComponent(input)}&bounds=${bounds}&size=${size}&key=${trackasiagl.accessToken}`;
                $.getJSON(url, function(data) {
                    if (data.predictions && data.predictions.length > 0) {
                        callback(data.predictions);
                    } else {
                        callback([]);
                    }
                });
            }
        };
        this.places = [];
        this.show_places = [];
        this.categories = {};
        this.tabs = [];
        this.all_shapes = [];
        this.trackasia_polylines = [];
        this.trackasia_polygons = [];
        this.trackasia_circles = [];
        this.trackasia_shape_events = [];
        this.trackasia_rectangles = [];
        this.per_page_value = 0;
        this.current_amenities = [];
        this.route_directions = [];
        this.search_area = '';
        this.markerClusterer = null;
        this.infowindow_marker = new trackasiagl.Popup();
        this.init();
    }

    MapLibreMaps.prototype = {

        init: function() {
            var map_obj = this;
            if (map_obj.map_data.map_property) {
                console.log('*********TRACKASIAP Debug Mode Output*********');
                console.log('Map ID =' + map_obj.map_data.map_property.map_id);
                if (map_obj.map_data.places) {
                    console.log('Total Locations=' + map_obj.map_data.places.length);
                }
                console.log('TRACKASIAP Object=');
                console.log(map_obj.map_data);
                console.log('*********TRACKASIAP Debug Mode End Output*********');
            }
            var center = [map_obj.settings.center_lng, map_obj.settings.center_lat];
            console.log('map_obj.settings', map_obj.settings);

            trackasiagl.accessToken = map_obj.settings.trackasia_api_key;
            map_obj.map = new trackasiagl.Map({
                container: map_obj.element,
                style: `https://maps.track-asia.com/v2/styles/v2/streets.json?key=${trackasiagl.accessToken}`,
                sprite: `https://maps.track-asia.com/v2/mapstyle/sprite?key=${trackasiagl.accessToken}`,
                center: center,
                zoom: map_obj.settings.zoom,
                scrollZoom: map_obj.settings.scroll_wheel,
                dragPan: map_obj.settings.draggable,
                accessToken: trackasiagl.accessToken,
                interactive: true
            })

            // Initialize Google Auto Suggest for the search box
            map_obj.google_auto_suggest($(".trackasia_auto_suggest"));

            map_obj.map.on('load', function() {
                map_obj.map_loaded();

                map_obj.create_markers();

                if (map_obj.settings.zoom_control) {
                    map_obj.map.addControl(new trackasiagl.ScaleControl());
                }

                if (map_obj.map_data.listing) {
                    map_obj.display_filters_listing();
                }
            });

            map_obj.map.on('zoom', () => {
                const currentZoom = map_obj.map.getZoom();
                $("#location_zoom").val(currentZoom);
            });
        },

        createMarker: function(place) {
            var map_obj = this;
            var map = map_obj.map;
            var placeLoc = [place.location.lng, place.location.lat];

            place.marker = new trackasiagl.Marker({
                color: place.location.icon
            }).setLngLat(placeLoc).addTo(map);

            place.marker.getElement().addEventListener('click', function() {
                if (map_obj.settings.map_infowindow_customisations === true)
                    map_obj.infowindow_marker.setHTML('<div class="trackasia_infowindow"><div class="trackasia_iw_content">' + place.name + '</div></div>');
                else
                    map_obj.infowindow_marker.setHTML(place.name);
                map_obj.infowindow_marker.setLngLat(placeLoc).addTo(map);
            });
            map_obj.current_amenities.push(place);
        },

        marker_bind: function(marker) {
            var map_obj = this;

            marker.on('dragend', function() {
                var position = marker.getLngLat();

                map_obj.geocoder.reverseGeocode(`${position.lat},${position.lng}`, function(result) {
                    if (result) {
                        $("#googlemap_address").val(result.formatted_address);
                        $(".google_city").val(map_obj.trackasia_finddata(result, 'administrative_area_level_3') || map_obj.trackasia_finddata(result, 'locality'));
                        $(".google_county").val(map_obj.trackasia_finddata(result, 'administrative_area_level_2'));
                        $(".google_state").val(map_obj.trackasia_finddata(result, "administrative_area_level_1"));
                        $(".google_country").val(map_obj.trackasia_finddata(result, "country") || "Việt Nam");
                    }
                });

                $(".google_latitude").val(position.lat);
                $(".google_longitude").val(position.lng);
            });
        },

        trackasia_finddata: function(result, type) {

            var component_name = "";

            for (var i = 0; i < result.address_components.length; ++i) {

                var component = result.address_components[i];

                $.each(component.types, function(index, value) {

                    if (value == type) {

                        component_name = component.long_name;

                    }

                });
            }

            return component_name;
        },

        google_auto_suggest: function(obj) {
            var map_obj = this;
        
            obj.each(function() {
                var current_input = this;
        
                // Create suggestions container if it doesn't exist
                if ($(current_input).next('.autocomplete-suggestions').length === 0) {
                    $(current_input).after('<div class="autocomplete-suggestions"></div>');
                }
        
                $(current_input).on('input', function() {
                    var input = $(this).val();
                    console.log('input', input);
                    if (input.length > 2) {
                        map_obj.geocoder.autocomplete(input, '', 5, function(predictions) {
                            var suggestions = predictions.map(function(prediction) {
                                console.log('prediction', prediction);
                                var location = prediction.geometry.location;
                                return `<div class="autocomplete-suggestion" data-place-id="${prediction.place_id}" data-lat="${location.lat}" data-lng="${location.lng}">${prediction.description}</div>`;
                            }).join('');
                            $(current_input).next('.autocomplete-suggestions').html(suggestions).show();
                        });
                    } else {
                        $(current_input).next('.autocomplete-suggestions').hide();
                    }
                });
        
                $(document).on('click', '.autocomplete-suggestion', function() {
                    var placeId = $(this).data('place-id');
                    console.log('placeId', placeId);
                    map_obj.geocoder.placeIdGeocode(placeId, function(result) {
                        if (result) {
                            var place = result.geometry.location;
                            console.log("result", result);
                            map_obj.map.setCenter([place.lng, place.lat]);
                            map_obj.map.setZoom(15);
                            $(".google_latitude").val(place.lat);
                            $(".google_longitude").val(place.lng);
                            $("#googlemap_address").val(result.formatted_address);
                            $(".google_city").val(map_obj.trackasia_finddata(result, 'administrative_area_level_3') || map_obj.trackasia_finddata(result, 'locality'));
                            $(".google_county").val(map_obj.trackasia_finddata(result, 'administrative_area_level_2'));
                            $(".google_state").val(map_obj.trackasia_finddata(result, "administrative_area_level_1"));
                            $(".google_country").val(map_obj.trackasia_finddata(result, "country") || "Việt Nam");
    
    
                            if (result.address_components) {
    
                                for (var i = 0; i < result.address_components.length; i++) {
    
                                    for (var j = 0; j < result.address_components[i].types.length; j++) {
    
                                        if (result.address_components[i].types[j] == "postal_code") {
    
                                            var trackasia_zip_code = result.address_components[i].long_name;
    
                                            $(".google_postal_code").val(trackasia_zip_code);
    
                                        }
    
                                    }
    
                                }
    
                            }

                            // Add marker to the map
                            var marker = new trackasiagl.Marker()
                                .setLngLat([place.lng, place.lat])
                                .addTo(map_obj.map);

                            // Store the marker in the map object for future reference
                            map_obj.current_amenities.push({
                                marker: marker,
                                location: place
                            });
                        }
                    });
                    $(current_input).val($(this).text());
                    $(current_input).next('.autocomplete-suggestions').hide();
                });
            });
        },

        open_infowindow: function(current_place) {
            var map_obj = this;

            $.each(this.map_data.places, function(key, place) {
                if (parseInt(place.id) == parseInt(current_place) && place.marker.isVisible()) {
                    map_obj.openInfoWindow(place);
                }
            });
        },

        place_info: function(place_id) {
            var place_obj;

            $.each(this.places, function(index, place) {
                if (parseInt(place.id) == parseInt(place_id)) {
                    place_obj = place;
                }
            });

            return place_obj;
        },

        event_listener: function(obj, type, func) {
            obj.on(type, func);
        },

        sortByPlace: function(order_by, data_type) {
            return function(a, b) {
                if (b[order_by] && a[order_by]) {
                    var a_val = a[order_by].toLowerCase();
                    var b_val = b[order_by].toLowerCase();
                    if (data_type == 'num') {
                        a_val = parseInt(a_val);
                        b_val = parseInt(b_val);
                    }
                    return ((a_val < b_val) ? -1 : ((a_val > b_val) ? 1 : 0));
                }
            }
        },

        sort_object_by_keyvalue: function(options, by, type, in_order) {
            var sortable = [];
            for (var key in options) {
                sortable.push(options[key]);
            }

            sortable.sort(this.sortByPlace(by, type));

            if (in_order == 'desc') {
                sortable.reverse();
            }

            return sortable;
        },

        create_filters: function() {
            var map_obj = this;
            var options = '';
            var filters = {};
            var places = this.map_data.places;
            var trackasia_listing_filter = this.map_data.listing;
        
            $.each(places, function(index, place) {
                if (typeof place.categories == 'undefined') {
                    place.categories = {};
                }
        
                $.each(place.categories, function(cat_index, category) {
                    if (typeof filters[category.type] == 'undefined') {
                        filters[category.type] = {};
                    }
        
                    if (category.name) {
                        if (category.extension_fields && category.extension_fields.cat_order) {
                            filters[category.type][category.name] = {
                                'id': category.id,
                                'order': category.extension_fields.cat_order,
                                'name': category.name
                            };
                        } else {
                            filters[category.type][category.name] = {
                                'id': category.id,
                                'order': 0,
                                'name': category.name
                            };
                        }
                    }
                });
            });
        
            var content = '',
                by = 'name',
                type = '',
                inorder = 'asc';
        
            if (map_obj.map_data.listing) {
                if (map_obj.map_data.listing.default_sorting) {
                    if (map_obj.map_data.listing.default_sorting.orderby == 'listorder') {
                        by = 'order';
                        type = 'num';
                        inorder = map_obj.map_data.listing.default_sorting.inorder;
                    }
                    inorder = map_obj.map_data.listing.default_sorting.inorder;
                }
            }
        
            $.each(filters, function(index, options) {
                if (trackasia_listing_filter.display_category_filter === true && index == "category") {
                    content += '<select data-filter="dropdown" data-name="category" name="place_' + index + '">';
                    content += '<option value="">' + trackasia_local.select_category + '</option>';
                    options = map_obj.sort_object_by_keyvalue(options, by, type, inorder);
                    $.each(options, function(name, value) {
                        content += "<option value='" + value.id + "'>" + value.name + "</option>";
                    });
                    content += '</select>';
                }
            });
        
            // Add event listener to trigger update_filters on change
            $(document).on('change', '[data-filter="dropdown"]', function() {
                map_obj.update_filters();
            });
        
            return content;
        },

        update_filters: function() {
            console.log('update_filters');
            var map_obj = this;
            var filters = {};

            var all_dropdowns = $(map_obj.container).find('[data-filter="dropdown"]');
            var all_checkboxes = $(map_obj.container).find('[data-filter="checklist"]:checked');
            var all_list = $(map_obj.container).find('[data-filter="list"].fc_selected');

            $.each(all_dropdowns, function(index, element) {
                if ($(this).val() != '') {
                    if (typeof filters[$(this).data('name')] == 'undefined') {
                        filters[$(this).data('name')] = [];
                    }
                    filters[$(this).data('name')].push($(this).val());
                }
            });

            $.each(all_checkboxes, function(index, element) {
                if (typeof filters[$(this).data('name')] == 'undefined') {
                    filters[$(this).data('name')] = [];
                }
                filters[$(this).data('name')].push($(this).val());
            });

            $.each(all_list, function(index, element) {
                if (typeof filters[$(this).data('name')] == 'undefined') {
                    filters[$(this).data('name')] = [];
                }
                filters[$(this).data('name')].push($(this).data('value').toString());
            });
            this.apply_filters(filters);
        },

        apply_filters: function(filters) {
            var map_obj = this;
            var selectedCategories = filters.category || [];
        
            $.each(map_obj.places, function(index, place) {
                if (!place.marker) {
                    return; // Skip if marker is not defined
                }
        
                var placeCategories = place.categories.map(function(category) {
                    return category.id;
                });
        
                var hasSelectedCategory = selectedCategories.some(function(category) {
                    return placeCategories.includes(category);
                });
        
                if (hasSelectedCategory || selectedCategories.length === 0) {
                    place.marker.getElement().style.display = 'block';
                } else {
                    place.marker.getElement().style.display = 'none';
                    place.infowindow.remove();
                }
            });
        },
        
        display_filters_listing: function() {
        
            if (this.map_data.listing) {
        
                var filter_content = '<div class="trackasia_filter_wrappers">' + this.display_filters() + '</div>';
                $(this.container).find(".trackasia_map_parent").after(filter_content);
        
            }
        
        },
        
        display_filters: function() {
        
            var hide_locations = this.map_data.listing.hide_locations;
        
            var content = '';
            content += '<div class="trackasia_before_listing">' + this.map_data.listing.listing_header + '</div>';
        
            content += '<div class="categories_filter">' + this.create_filters() + '<div data-container="trackasiap-filters-container"></div>';
            content += '</div>';
        
            return content;
        },
        
        map_loaded: function() {
        
            var map_obj = this;
        
            map_obj.map.on('load', function() {
        
                // Add any additional layers or sources here if needed
        
            });
        
            if (map_obj.settings.center_by_nearest === true) {
                map_obj.center_by_nearest();
            }
            if (map_obj.settings.close_infowindow_on_map_click === true) {
                map_obj.map.on("click", function(event) {
                    $.each(map_obj.places, function(key, place) {
                        if (place.infowindow) {
                            place.infowindow.remove();
                        }
                        place.marker.getElement().classList.remove('bounce');
                    });
                });
            }
        
            // Implement custom popup logic with trackasiagl.Popup
            // Additional customizations can be added here
        
            if (map_obj.settings.map_infowindow_customisations === true) {
                // Implement custom popup styles if needed
                // This may involve adding custom HTML/CSS for popups
            }
        
        },
        
        resize_map: function() {
            var map_obj = this;
            map_obj.map.resize();
        },
        
        responsive_map: function() {
        
            var map_obj = this;
        
            map_obj.map.on('resize', function() {
                var zoom = map_obj.map.getZoom();
                var center = map_obj.map.getCenter();
                map_obj.map.resize();
                map_obj.map.setZoom(zoom);
                map_obj.map.setCenter(center);
            });
        
        },
        
        create_markers: function() {
        
            var map_obj = this;
            var places = map_obj.map_data.places;
            var remove_keys = [];
        
            $.each(places, function(key, place) {
        
                if (place.location.lat && place.location.lng) {
                    console.log('create_markers place', place);
                    if (typeof place.categories == 'undefined') {
                        place.categories = {};
                    }
                    var marker = new trackasiagl.Marker({
                        element: document.createElement('div')
                    });

                    var markerElm = marker.getElement();
                    markerElm.className = 'custom-marker'; // Add custom class for styling
                    markerElm.style.backgroundImage = `url('${place.location.icon}')`;
                    markerElm.style.backgroundSize = 'cover';
                    markerElm.style.width = '30px';
                    markerElm.style.height = '30px';
                    markerElm.style.borderRadius = '50%';
                    markerElm.style.cursor = 'pointer';

                    marker.setLngLat([parseFloat(place.location.lng), parseFloat(place.location.lat)]);
                    marker.addTo(map_obj.map);
        
                    if (map_obj.settings.infowindow_drop_animation === true) {
                        markerElm.classList.add('drop-animation'); // Define CSS for drop animation
                    }
        
                    if (map_obj.settings.infowindow_filter_only === true) {
                        markerElm.style.display = 'none';
                    }
        
                    // bind event to marker
                    if (map_obj.map_data.page == 'edit_location')
                        map_obj.marker_bind(marker);
        
                    var location_categories = [];
                    if (typeof place.categories != 'undefined') {
                        for (var cat in place.categories) {
                            location_categories.push(place.categories[cat].name);
                        }
                    }
        
                    var content = '';
                    // replace infowindow content.
                    var marker_image = '';
        
                    if (place.source == 'post') { 
                        marker_image = place.location.extra_fields.post_featured_image; 
                    } else {
                        marker_image = place.location.marker_image;
                    }
        
                    // Replace placeholders with actual content
                    var temp_listing_placeholder = '';
                    var post_info_class = 'fc-infowindow-';
                    if (place.source == 'post') {
                        temp_listing_placeholder = map_obj.settings.infowindow_geotags_setting;
                        post_info_class = 'trackasia_infowindow_post fc-item-' + map_obj.settings.infowindow_post_skin.name;
                    } else {
                        temp_listing_placeholder = map_obj.settings.infowindow_setting;
                        if (map_obj.map_data.page != 'edit_location' && map_obj.settings.infowindow_skin)
                            post_info_class = 'fc-infowindow-' + map_obj.settings.infowindow_skin.name;
                    }
        
                    if (typeof temp_listing_placeholder == 'undefined') {
                        temp_listing_placeholder = place.content;
                    }
        
                    var replaceData = {
                        "{marker_id}": place.id,
                        "{marker_title}": place.title,
                        "{marker_address}": place.address,
                        "{marker_latitude}": place.location.lat,
                        "{marker_longitude}": place.location.lng,
                        "{marker_city}": place.location.city,
                        "{marker_county}": place.location.county,
                        "{marker_state}": place.location.state,
                        "{marker_country}": place.location.country,
                        "{marker_postal_code}": place.location.postal_code,
                        "{marker_zoom}": place.location.zoom,
                        "{marker_icon}": place.location.icon,
                        "{marker_category}": location_categories.join(','),
                        "{marker_message}": place.content,
                        "{marker_image}": marker_image,
                        "{get_directions_link}": 'https://www.google.com/maps/place/' + parseFloat(place.location.lat) + ',' + parseFloat(place.location.lng),
                    };
        
                    // Add extra fields of locations
                    if (typeof place.location.extra_fields != 'undefined') {
                        for (var extra in place.location.extra_fields) {
                            if (!place.location.extra_fields[extra]) {
                                replaceData['{' + extra + '}'] = '';
                            } else {
                                replaceData['{' + extra + '}'] = place.location.extra_fields[extra];
                            }
                        }
                    }
        
                    temp_listing_placeholder = temp_listing_placeholder.replace(/{#if (.*?)}([\s\S]*?){\/if}/g, function(match, p1, p2) {
                        return replaceData['{' + p1 + '}'] ? p2 : '';
                    });
        
                    temp_listing_placeholder = temp_listing_placeholder.replace(/{[^{}]+}/g, function(match) {
                        if (match in replaceData) {
                            return (replaceData[match]);
                        } else {
                            return ("");
                        }
                    });
        
                    content = temp_listing_placeholder;
        
                    var trackasia_infowindow_class = '';
                    var trackasia_iw_content = '';
                    if (map_obj.map_data.map_options.infowindow_skin != '') {
                        if (map_obj.map_data.map_options.infowindow_skin && map_obj.map_data.map_options.infowindow_skin.name && map_obj.map_data.map_options.infowindow_skin.name == 'basic') {
                            trackasia_infowindow_class = '';
                            trackasia_iw_content = '';
                        } else {
                            trackasia_infowindow_class = 'class="trackasia_infowindow ' + post_info_class + '"';
                            trackasia_iw_content = 'class="trackasia_iw_content"';
                        }
                    }
        
                    if (content === "") {
                        if (map_obj.settings.map_infowindow_customisations === true && map_obj.settings.show_infowindow_header === true)
                            content = '<div ' + trackasia_infowindow_class + '><div class="trackasia_iw_head"><div class="trackasia_iw_head_content">' + place.title + '</div></div><div class="trackasia_iw_content">' + place.content + '</div></div>';
                        else
                            content = '<div ' + trackasia_infowindow_class + '><div ' + trackasia_iw_content + '>' + place.content + '</div></div>';
                    } else {
                        if (map_obj.settings.map_infowindow_customisations === true && map_obj.settings.show_infowindow_header === true)
                            content = '<div ' + trackasia_infowindow_class + '><div class="trackasia_iw_head"><div class="trackasia_iw_head_content">' + place.title + '</div></div><div class="trackasia_iw_content">' + content + '</div></div>';
                        else
                            content = '<div ' + trackasia_infowindow_class + '><div ' + trackasia_iw_content + '>' + content + '</div></div>';
                    }
        
                    place.infowindow_data = content;
                    place.infowindow = new trackasiagl.Popup({
                        // offset: 25,
                        closeButton: true,
                        closeOnClick: false,
                        maxWidth: '',
                    }).setHTML(place.infowindow_data);
        
                    if (place.location.infowindow_default_open === true) {
                        map_obj.openInfoWindow(place);
                    } else if (map_obj.settings.default_infowindow_open === true) {
                        map_obj.openInfoWindow(place);
                    }
        
                    var on_event = map_obj.settings.infowindow_open_event;
                    var bounce_on_event = map_obj.settings.infowindow_bounce_animation;
        
                    markerElm.addEventListener(on_event, function() {
                        map_obj.openInfoWindow(place);
                        if (bounce_on_event == 'click') {
                            markerElm.classList.toggle('bounce');
                        }
                    });
        
                    if (bounce_on_event == 'mouseover' && on_event != 'mouseover') {
                        markerElm.addEventListener('mouseover', function() {
                            markerElm.classList.add('bounce');
                        });
        
                        markerElm.addEventListener('mouseout', function() {
                            markerElm.classList.remove('bounce');
                        });
                    }
        
                    if (bounce_on_event != '') {
                        place.infowindow.on('close', function() {
                            markerElm.classList.remove('bounce');
                        });
                    }

                    place.marker = marker;
        
                    map_obj.places.push(place);
                } else {
                    remove_keys.push(key);
                }
            });
            $.each(remove_keys, function(index, value) {
                places.splice(value, 1);
            });
        
        },

        openInfoWindow: function(place) {
            var map_obj = this;
        
            place.infowindow.addTo(map_obj.map);
            place.infowindow.setLngLat([parseFloat(place.location.lng), parseFloat(place.location.lat)]);
        
            if (place.location.onclick_action == "post") {
                if (place.location.open_new_tab == 'yes')
                    window.open(place.location.redirect_permalink, '_blank');
                else
                    window.open(place.location.redirect_permalink, '_self');
            } else if (place.location.onclick_action == "custom_link") {
                if (place.location.open_new_tab == 'yes')
                    window.open(place.location.redirect_custom_link, '_blank');
                else
                    window.open(place.location.redirect_custom_link, '_self');
            }
            // Additional logic if needed
        },
        
        get_current_location: function(success_func, error_func) {
        
            var map_obj = this;
        
            if (typeof map_obj.user_location == 'undefined') {
        
                navigator.geolocation.getCurrentPosition(function(position) {
                    map_obj.user_location = [position.coords.longitude, position.coords.latitude];
        
                    if (success_func)
                        success_func(map_obj.user_location);
        
                }, function(ErrorPosition) {
        
                    if (error_func)
                        error_func(ErrorPosition);
        
                }, {
                    enableHighAccuracy: true,
                    timeout: 5000,
                    maximumAge: 0
                });
            } else {
                if (success_func)
                    success_func(map_obj.user_location);
            }
        },
        
        toggle_bounce: function(marker) {
            // TrackasiaGL does not have built-in bounce animation
            // Implement custom bounce animation using CSS or a library
            if (marker.getElement().classList.contains('bounce')) {
                marker.getElement().classList.remove('bounce');
            } else {
                marker.getElement().classList.add('bounce');
            }
        },
    };

    $.fn.maps = function(options, places) {
    
        this.each(function() {
    
            if (!$.data(this, "trackasia_maps")) {
                $.data(this, "trackasia_maps", new MapLibreMaps(this, options, places));
            }
    
        });
        // chain jQuery functions
        return this;
    };

}(jQuery, window, document));

function trackasiapInitMap() {

    jQuery(document).ready(function($) {

        if ($.fn.maps) {

            $(".trackasia_map").each(function() {

                var mapId = $(this).data("map-id");
                var mapVarName = 'mapdata' + mapId;
                var mapData = window.trackasiap && window.trackasiap[mapVarName];
                if (mapData) {
                    $("#map" + mapId).maps(mapData).data("trackasia_maps");
                } else {
                    console.warn("Map data missing for map : " + mapId);
                }

            });
        }
    });
}
