<?php
/**
 * Class: TRACKASIAP_Model_Settings
 * @author TrackAsia Code <support@trackasia.com>
 * @version 1.0.0
 * @package Maps
 */

if ( ! class_exists( 'TRACKASIAP_Model_Settings' ) ) {

	/**
	 * Setting model for Plugin Options.
	 * @package Maps
	 * @author TrackAsia Code <support@trackasia.com>
	 */
	class TRACKASIAP_Model_Settings extends FlipperCode_Model_Base {
		/**
		 * Intialize Backup object.
		 */
		function __construct() {
		}
		/**
		 * Admin menu for Settings Operation
		 * @return array Admin menu navigation(s).
		 */
		function navigation() {
			return array(
				'trackasia_manage_settings' => esc_html__( 'Plugin Settings', 'wp-trackasia-maps' ),
			);
		}
		/**
		 * Add or Edit Operation.
		 */
		function save() {
			
			global $_POST;
			
			//Permission Verification
			if ( ! current_user_can('administrator') )
			die( 'You are not allowed to save changes!' );
			
			//Nonce Verification
			if( !isset( $_REQUEST['_wpnonce'] ) || ( isset( $_REQUEST['_wpnonce'] ) && empty($_REQUEST['_wpnonce']) ) )
			die( 'You are not allowed to save changes!' );
			if ( isset( $_REQUEST['_wpnonce'] ) && ! wp_verify_nonce( $_REQUEST['_wpnonce'], 'trackasiap-nonce' ) )
			die( 'You are not allowed to save changes!' );

			//Check Validations
			$this->verify( $_POST );

			if ( is_array( $this->errors ) and ! empty( $this->errors ) ) {
				$this->throw_errors();
			}
			$extra_fields = array();
			if ( isset( $_POST['location_extrafields'] ) ) {
				foreach ( $_POST['location_extrafields'] as $index => $label ) {
					if ( $label != '' ) {
						$extra_fields[$index] = sanitize_text_field( wp_unslash( $label ) );
					}
				}
			}

			$meta_hide = array();
			if ( isset( $_POST['trackasia_allow_meta'] ) ) {
				foreach ( $_POST['trackasia_allow_meta'] as $index => $label ) {
					if ( $label != '' ) {
						$meta_hide[$index] = sanitize_text_field( wp_unslash( $label ) );
					}
				}
			}
			update_option( 'trackasia_language',sanitize_text_field( wp_unslash( $_POST['trackasia_language'] ) ) );
			update_option( 'trackasia_api_key',sanitize_text_field( wp_unslash( $_POST['trackasia_api_key'] ) ) );
			update_option( 'trackasia_scripts_place',sanitize_text_field( wp_unslash( $_POST['trackasia_scripts_place'] ) ) );
			update_option( 'trackasia_location_extrafields', serialize(  $extra_fields  ) );
			update_option( 'trackasia_allow_meta', serialize(  $meta_hide  ));
			
			$trackasia_settings = get_option( 'trackasia_settings', true );

			if ( ! is_array( $trackasia_settings ) ) {
				$trackasia_settings = array();
			}

			if ( isset( $_POST['trackasia_auto_fix'] ) ) {

				$trackasia_settings['trackasia_auto_fix']         = sanitize_text_field( wp_unslash( $_POST['trackasia_auto_fix'] ) );
			} else {
				$trackasia_settings['trackasia_auto_fix']         = '';
			}

			if ( isset( $_POST['trackasia_debug_mode'] ) ) {
				$trackasia_settings['trackasia_debug_mode']             = sanitize_text_field( wp_unslash( $_POST['trackasia_debug_mode'] ) );
			} else {
				$trackasia_settings['trackasia_debug_mode']             = '';
			}

			if ( isset( $_POST['trackasia_gdpr'] ) ) {
				$trackasia_settings['trackasia_gdpr']             = sanitize_text_field( wp_unslash( $_POST['trackasia_gdpr'] ) );
			} else {
				$trackasia_settings['trackasia_gdpr']             = '';
			}

			if ( isset( $_POST['trackasia_gdpr_show_placeholder'] ) ) {
				$trackasia_settings['trackasia_gdpr_show_placeholder']             = sanitize_text_field( wp_unslash( $_POST['trackasia_gdpr_show_placeholder'] ) );
			} else {
				$trackasia_settings['trackasia_gdpr_show_placeholder']             = '';
			}

			// $trackasia_settings['trackasia_gdpr_msg']         = wp_unslash( $_POST['trackasia_gdpr_msg'] );

			$trackasia_settings = apply_filters('trackasia_plugin_settings',$trackasia_settings);
			update_option( 'trackasia_settings', $trackasia_settings );

			$response['success'] = esc_html__( 'Setting(s) were saved successfully.','wp-trackasia-maps' );
			return $response;

		}
	}
}
