<?php
/**
 * This class used to manage settings page in backend.
 * @author TrackAsia Code <support@trackasia.com>
 * @version 1.0.0
 * @package Maps
 */

$trackasia_settings = get_option( 'trackasia_settings', true );
$api_key = get_option( 'trackasia_api_key' );

if ( $api_key != '' ) {
	$google_map_api = 'https://maps.google.com/maps/api/js?key=' . $api_key . '&callback=trackasiapInitMap&libraries=geometry,places,drawing&language=en';
} else {
	$google_map_api = 'https://maps.google.com/maps/api/js?libraries=geometry,places,drawing&callback=trackasiapInitMap&language=en';
}


// wp_enqueue_script('trackasiap-backend-google-api', $google_map_api, array('trackasiap-map'), TRACKASIAP_VERSION);

$form  = new TRACKASIAP_Template();
$form->form_action = esc_url ( add_query_arg( 'page', 'trackasia_manage_settings', admin_url ('admin.php') )  );
$form->set_header( esc_html__( 'General Plugin Settings', 'wp-trackasia-maps' ), $response, $enable_accordion = false );

$form->add_element( 'group', 'trackasia_general_settings', array(
	'value' => esc_html__( 'General Plugin Settings', 'wp-trackasia-maps' ),
	'before' => '<div class="fc-12">',
	'after' => '</div>',
	// 'tutorial_link' => 'https://account.track-asia.vn/'
));

$referrer = home_url();
$referrer_two = '*'.$_SERVER['HTTP_HOST'].'/*';

$link = '<a href="https://account.track-asia.vn/" target="_blank">'.esc_html__("Create API Key","wp-trackasia-maps").'</a>'; 

$form->set_col( 2 );

$form->add_element('text','trackasia_api_key',array(
	'lable' => esc_html__( 'TrackAsia Maps API Key','wp-trackasia-maps' ),
	'value' => get_option( 'trackasia_api_key' ),
	'before' => '<div class="fc-6"> <div class="trackasia_apitest"></div>',
	'after'  => '</div>',
	'desc'  => sprintf( esc_html__( 'If you don\'t have an API key, create one here. %1$s', 'wp-trackasia-maps' ), $link ),
));

if ( get_option( 'trackasia_api_key' ) == '' ) {

	$generate_link = '<a href="https://account.track-asia.vn/" class="trackasia_map_key_missing trackasia_key_btn fc-btn fc-btn-default btn-lg" target="_blank" >' . esc_html__( 'Generate API Key', 'wp-trackasia-maps' ) . '</a>';

	$form->add_element(
		'html', 'trackasia_key_btn', array(
			'html'   => $generate_link,
			'before' => '<div class="fc-2">',
			'after'  => '</div>',
		)
	);

} else {

	$generate_link = '<a href="javascript:void(0);" class="trackasia_check_key fc-btn fc-btn-default btn-lg" >' . esc_html__( 'Test API Key', 'wp-trackasia-maps' ) . '</a>';

	$form->add_element(
		'html', 'trackasia_key_btn', array(
			'html'   => $generate_link,
			'before' => '<div class="fc-2">',
			'after'  => '</div>',
		)
	);
}

$form->set_col( 1 );

$guide_link = '<a href="https://track-asia.com/vn/#contact" target="_blank">'.esc_html__("contact.",'wp-trackasia-maps').'</a>'; 

$form->add_element(
	'html', 'trackasia_map_preview', array(
		'lable' => esc_html__( 'Map Preview', 'wp-trackasia-maps' ),
		'html'   => "<div id='trackasia_map_preview' style='width:100%;height:300px'></div>",
		'before' => '<div class="fc-6">',
		'after'  => '</div>',
		'desc'   => sprintf(esc_html__( 'If map is not visible then please check the javascript console and report the error to us at %1$s', 'wp-trackasia-maps' ), $guide_link)
	)
);

$language = array(
'vi' => esc_html__( 'VIETNAMESE', 'wp-trackasia-maps' ),
'en' => esc_html__( 'ENGLISH', 'wp-trackasia-maps' ),
);

$form->add_element( 'select', 'trackasia_language', array(
	'lable' => esc_html__( 'Map Language', 'wp-trackasia-maps' ),
	'current' => get_option( 'trackasia_language' ),
	'desc' => esc_html__( 'Choose the language for google map. The default language is Vietnamese.', 'wp-trackasia-maps' ),
	'options' => $language,
	'before' => '<div class="fc-6">',
	'after' => '</div>',
));

$form->add_element( 'radio', 'trackasia_scripts_place', array(
	'lable' => esc_html__( 'Include Scripts in ', 'wp-trackasia-maps' ),
	'radio-val-label' => array( 'header' => esc_html__( 'Header','wp-trackasia-maps' ),'footer' => esc_html__( 'Footer (Recommended)','wp-trackasia-maps' ) ),
	'current' => get_option( 'trackasia_scripts_place' ),
	'class' => 'chkbox_class',
	'default_value' => 'footer',
));


$form->add_element('submit','trackasia_save_settings',array(
	'value' => esc_html__( 'Save Settings','wp-trackasia-maps' ),
	));
$form->add_element('hidden','operation',array(
	'value' => 'save',
	));
$form->add_element('hidden','page_options',array(
	'value' => 'trackasia_api_key,trackasia_scripts_place',
	));
$form->render();
$map_data['map_options'] = array(
	'center_lat' =>  '16.07526',
	'center_lng' =>  '108.224212',
);
$map_data['map_property'] = array('map_id' => 1);
?>

<script type="text/javascript">
jQuery(document).ready(function($) {
    var map = $("#trackasia_map_preview").maps("<?php echo esc_js( base64_encode( wp_json_encode( $map_data ) ) ); ?>").data('trackasia_maps');
});
</script>
