<?php
/*
Plugin Name: TrackAsia Maps
Plugin URI: https://track-asia.com/
Description: A fully customizable WordPress Plugin for TrackAsia Maps. Create unlimited TrackAsia Maps Shortcodes, assign unlimited locations with custom infowindow messages and add to pages, posts and widgets.
Author: trackasia
Author URI: https://track-asia.com/
Version: 1.0.0
Text Domain: wp-trackasia-maps
Domain Path: /lang
*/

if (! defined('ABSPATH')) {
	die('You are not allowed to call this page directly.');
}

if (! class_exists('FC_TrackAsia_Maps')) {

	/**
	 * Main plugin class
	 * @author TrackAsia Code <support@trackasia.com>
	 * @package Maps
	 */
	class FC_TrackAsia_Maps
	{
		/**
		 * List of Modules.
		 * @var array
		 */
		private $modules = array();
		/**
		 * Intialize variables, files and call actions.
		 * @var array
		 */
		public function __construct()
		{

			$this->trackasia_define_constants();
			$this->trackasia_load_files();

			register_activation_hook(__FILE__, array($this, 'trackasia_plugin_activation'));
			register_deactivation_hook(__FILE__, array($this, 'trackasia_plugin_deactivation'));
			if (is_multisite()) {

				add_action('wpmu_new_blog',    array($this, 'trackasia_on_blog_new_generate'), 10, 6);
				add_filter('wpmu_drop_tables', array($this, 'trackasia_on_blog_delete'));
			}
			add_action('plugins_loaded',     array($this, 'trackasia_load_plugin_languages'));
			add_action('init',               array($this, 'trackasia_init'));
			// add_action( 'widgets_init',       array( $this, 'trackasia_google_map_widget' ) );

		}

		/**
		 * Call WordPress hooks.
		 */
		function trackasia_init()
		{

			add_action('media_upload_ell_insert_gmap_tab', array($this, 'trackasia_google_map_media_upload_tab'));
			add_action('wp_enqueue_scripts',               array($this, 'trackasia_frontend_scripts'));
			add_shortcode('put_trackasia',                      array($this, 'trackasia_show_location_in_map'));
			add_shortcode('display_map',                   array($this, 'trackasia_display_map'));
			add_filter('fc-dummy-placeholders', 		    array($this, 'trackasia_apply_placeholders'));

			if (is_admin()) {
				// add_action( 'admin_head', 				  	array( $this, 'trackasia_customizer_font_family' ) );
				add_action('admin_menu',                   array($this, 'trackasia_create_menu'));
				add_filter('media_upload_tabs',            array($this, 'trackasia_google_map_tabs_filter'));
				add_action('admin_enqueue_scripts',        array($this, 'trackasia_overview_page_styles'));
				add_action('admin_head', array($this, 'trackasia_custom_css_admin_head'));
				// add_action('wp_ajax_trackasia_hide_buy_notice', array($this, 'trackasia_hide_buy_notice'));
				// add_action('wp_ajax_nopriv_trackasia_hide_buy_notice', array($this, 'trackasia_hide_buy_notice'));
				// add_action('wp_ajax_trackasia_hide_sample_notice', array($this, 'trackasia_hide_sample_notice'));
				// add_action('wp_ajax_nopriv_trackasia_hide_sample_notice', array($this, 'trackasia_hide_sample_notice'));
				// add_action('wp_ajax_trackasia_submit_uninstall_reason_action', array($this, 'trackasia_submit_uninstall_reason_action_perform'));
				// add_action('wp_ajax_nopriv_trackasia_submit_uninstall_reason_action', array($this, 'trackasia_submit_uninstall_reason_action'));
			}
		}

		function trackasia_submit_uninstall_reason_action_perform()
		{


			global  $wp_version, $current_user;
			wp_verify_nonce($_REQUEST['trackasia_ajax_nonce'], 'trackasia_ajax_nonce');
			$reason_id = isset($_REQUEST['reason_id']) ? stripcslashes(sanitize_text_field($_REQUEST['reason_id'])) : '';
			$basename  = isset($_REQUEST['plugin']) ? stripcslashes(sanitize_text_field($_REQUEST['plugin'])) : '';

			if (empty($reason_id) || empty($basename)) {
				exit;
			}

			$reason_info = isset($_REQUEST['reason_info']) ? stripcslashes(sanitize_textarea_field($_REQUEST['reason_info'])) : '';
			if (!empty($reason_info)) {
				$reason_info = substr($reason_info, 0, 255);
			}
			$is_anonymous = isset($_REQUEST['is_anonymous']) && 1 == $_REQUEST['is_anonymous'];


			$options = array(
				'product'     => 'TrackAsia Maps Plugin',
				'reason_id'   => $reason_id,
				'reason_info' => $reason_info,
			);

			if (!$is_anonymous) {
				$options['url']                  = get_site_url();
				$options['wp_version']           = $wp_version;
				$options['plugin_version']       = TRACKASIAP_VERSION;
				$options['email'] = $current_user->data->user_email;
			}

			wp_remote_post(
				"https://weplugins.com/wp-json/weplugins/v1/plugin-deactivate",
				array(
					'method'  => 'POST',
					'body'    => $options,
					'timeout' => 15,
				)
			);
			exit;
		}

		/**
		 * Register WP Google Map Widget
		 */
		// function trackasia_google_map_widget() { register_widget( 'TRACKASIAP_Google_Map_Widget_Class' ); } 

		function trackasia_customizer_font_family()
		{

			if (isset($_GET['doaction']) and 'edit' == $_GET['doaction'] and isset($_GET['map_id'])) {

				$modelFactory = new TRACKASIAP_Model();
				$map_obj      = $modelFactory->create_object('map');
				$styles_and_scripts = $map_obj->get_map_customizer_style();
				$font_families   = $styles_and_scripts['font_families'];
				$fc_skin_styles  = $styles_and_scripts['fc_skin_styles'];
				if (! empty($fc_skin_styles)) {
					echo '<style>' . esc_html($fc_skin_styles) . '</style>';
				}
				if (! empty($font_families)) {
					$font_families = array_unique($font_families);
?>
				<?php }
			}
		}


		function trackasia_go_pro_link($actions)
		{

			$actions['settings'] = '<a href="' . admin_url('admin.php?page=trackasia_manage_settings') . '">' . esc_html__('Settings', 'wp-trackasia-maps') . '</a>';
			$actions['go_pro'] = '<a style="color:#2ea100;" target = "_blank" href="https://www.wpmapspro.com/?utm_source=wordpress&utm_medium=liteversion&utm_campaign=freemium&utm_id=freemium">' . esc_html__('Pro Version', 'wp-trackasia-maps') . '</a>';


			return $actions;
		}

		function trackasia_custom_css_admin_head()
		{

			$modelFactory = new TRACKASIAP_Model();
			$location_obj = $modelFactory->create_object('location');
			$locations = $location_obj->fetch();
			if (is_array($locations) && count($locations) > 0) { ?>
				<style>
					.wp-list-table .locations {
						margin-top: 40px;
					}
				</style> <?php
								}
							}

							/**
							 * Eneque script at frontend.
							 */
							function trackasia_frontend_scripts()
							{

								global $post;

								$trackasia_settings = get_option('trackasia_settings', true);

								$auto_fix = '';

								if (isset($trackasia_settings['trackasia_auto_fix']) && !empty($trackasia_settings['trackasia_auto_fix'])) {

									$auto_fix = $trackasia_settings['trackasia_auto_fix'];

									if ($auto_fix == 'true') {
										wp_enqueue_script('jquery');
									}
								}

								// Check if the current page content has the shortcode [put_trackasia id=X]
								if (is_a($post, 'WP_Post') && has_shortcode($post->post_content, 'put_trackasia') && preg_match('/\[put_trackasia id=\d+\]/', $post->post_content)) {
									wp_enqueue_style('trackasiap-frontend_css', TRACKASIAP_CSS . 'frontend.css');
									wp_enqueue_script('jquery');
								}

								$language = get_option('trackasia_language');

								if ($language == '')
									$language = 'vi';


								$key = get_option('trackasia_api_key', 'public_key');
								$key = $key != "" ? $key : 'public_key';
								$trackasia_map_api = 'https://maps.track-asia.com/api/v2/js?key=' . $key . '&callback=trackasiapInitMap&libraries=geometry,places&language=' . $language;
								$trackasia_map_css = 'https://maps.track-asia.com/api/v2/css?key='. $key;

								$where = get_option('trackasia_scripts_place');

								if ($where == 'header') {
									$where = false;
								} else {
									$where = true;
								}

								$trackasia_local = array();
								$trackasia_local['all_location'] = esc_html__('All', 'wp-trackasia-maps');
								$trackasia_local['show_locations'] = esc_html__('Show Locations', 'wp-trackasia-maps');
								$trackasia_local['sort_by'] = esc_html__('Sort by', 'wp-trackasia-maps');
								$trackasia_local['trackasia_not_working'] = esc_html__('Not working...', 'wp-trackasia-maps');
								$trackasia_local['select_category'] = esc_html__('Select Category', 'wp-trackasia-maps');
								$trackasia_local['place_icon_url'] = TRACKASIAP_ICONS;
								$trackasia_local['trackasia_assets'] = TRACKASIAP_JS;

								$scripts = array();

								$scripts[] = array(
									'handle'  => 'trackasiap-google-map-main',
									'src'   => TRACKASIAP_JS . 'maps.js',
									'deps'    => array(),
								);

								$scripts[] = array(
									'handle'  => 'trackasiap-trackasia-gl',
									'src'   => $trackasia_map_api,
									'deps'    => array('trackasiap-google-map-main'),
								);

								if ($scripts) {
									foreach ($scripts as $script) {
										if ($auto_fix == 'true') {
											wp_enqueue_script($script['handle'], $script['src'], $script['deps'], TRACKASIAP_VERSION, $where);
										} else {
											wp_register_script($script['handle'], $script['src'], $script['deps'], TRACKASIAP_VERSION, $where);
										}
									}
								}

								wp_localize_script('trackasiap-google-map-main', 'trackasia_local', $trackasia_local);
								wp_enqueue_style('trackasia-gl-css', $trackasia_map_css);
							}

							/**
							 * Display google maps at the frontend using put_trackasiap shortcode.
							 * @param  array  $atts   Map Options.
							 * @param  string $content Content.
							 */
							function trackasia_show_location_in_map($atts, $content = null)
							{

								if (!isset($atts['id']) || empty($atts['id']))
									return '';

								$atts['id'] = sanitize_text_field($atts['id']);

								if (!is_numeric($atts['id'])) {
									return '';
								}

								$atts['id'] = intval($atts['id']);
								if ($atts['id'] <= 0) {
									return '';
								}

								try {
									$factoryObject = new TRACKASIAP_Controller();
									$viewObject = $factoryObject->create_object('shortcode');
									$output = $viewObject->display('put-trackasia', $atts);
									return $output;
								} catch (Exception $e) {
									echo wp_kses_post(TRACKASIAP_Template::show_message(array('error' => $e->getMessage())));
								}
							}
							/**
							 * Display google maps at the frontend using display_map shortcode.
							 * @param  array $atts    Map Options.
							 */
							function trackasia_display_map($atts)
							{

								try {
									$factoryObject = new TRACKASIAP_Controller();
									$viewObject = $factoryObject->create_object('shortcode');
									$output = $viewObject->display('display-map', $atts);
									return $output;
								} catch (Exception $e) {
									echo wp_kses_post(TRACKASIAP_Template::show_message(array('error' => $e->getMessage())));
								}
							}
							/**
							 * Process slug and display view in the backend.
							 */
							function trackasia_processor()
							{

								$return = '';
								if (isset($_GET['page'])) {
									$page = sanitize_key(wp_unslash($_GET['page']));
								} else {
									$page = 'trackasia_view_overview';
								}

								$pageData = explode('_', $page);
								$obj_type = $pageData[2];
								$obj_operation = $pageData[1];

								if (count($pageData) < 3) {
									die('Cheating!');
								}

								try {
									if (count($pageData) > 3) {
										$obj_type = $pageData[2] . '_' . $pageData[3];
									}

									$factoryObject = new TRACKASIAP_Controller();
									$viewObject = $factoryObject->create_object($obj_type);
									$viewObject->display($obj_operation);
								} catch (Exception $e) {
									echo wp_kses_post(TRACKASIAP_Template::show_message(array('error' => $e->getMessage())));
								}
							}
							/**
							 * Create backend navigation.
							 */
							function trackasia_create_menu()
							{

								global $navigations;

								$pagehook1 = add_menu_page(
									esc_html__('TRACK ASIA MAPS', 'wp-trackasia-maps'),
									esc_html__('TRACK ASIA MAPS', 'wp-trackasia-maps'),
									'trackasia_admin_overview',
									TRACKASIAP_SLUG,
									array($this, 'trackasia_processor'),
									esc_url(TRACKASIAP_IMAGES . 'trackasia.png')
								);

								if (current_user_can('manage_options')) {
									$role = get_role('administrator');
									$role->add_cap('trackasia_admin_overview');
								}

								$this->trackasia_load_modules_menu();

								add_action('load-' . $pagehook1, array($this, 'trackasia_backend_scripts'));
							}
							/**
							 * Read models and create backend navigation.
							 */
							function trackasia_load_modules_menu()
							{

								$modules = $this->modules;
								$pagehooks = array();
								if (is_array($modules)) {
									foreach ($modules as $module) {

										$object = new $module;

										if (method_exists($object, 'navigation')) {

											if (! is_array($object->navigation()))
												continue;

											foreach ($object->navigation() as $nav => $title) {

												if (current_user_can('manage_options') && is_admin()) {
													$role = get_role('administrator');
													$role->add_cap($nav);
												}

												$pagehooks[] = add_submenu_page(
													TRACKASIAP_SLUG,
													$title,
													$title,
													$nav,
													$nav,
													array($this, 'trackasia_processor')
												);
											}
										}
									}
								}

								if (is_array($pagehooks)) {

									foreach ($pagehooks as $key => $pagehook) {
										add_action('load-' . $pagehooks[$key], array($this, 'trackasia_backend_scripts'));
									}
								}
							}


							/**
							 * Eneque scripts in the backend.
							 */
							function trackasia_backend_scripts()
							{

								$screen = get_current_screen();

								$language = get_option( 'trackasia_language' );

								if ( $language == '' )
									$language = 'vi';

								$key = get_option('trackasia_api_key', 'public_key');
								$key = $key != "" ? $key : 'public_key';
								$trackasia_map_api = 'https://maps.track-asia.com/api/v2/js?key=' . $key . '&callback=trackasiapInitMap&libraries=geometry,places&language=' . $language;
								$trackasia_map_css = 'https://maps.track-asia.com/api/v2/css?key='. $key;

								wp_enqueue_style('thickbox');
								wp_enqueue_style('wp-color-picker');
								$wp_scripts = array('jQuery', 'thickbox', 'wp-color-picker');

								if ($wp_scripts) {
									foreach ($wp_scripts as $wp_script) {
										wp_enqueue_script($wp_script);
									}
								}

								$scripts = array();

								$scripts[] = array(
									'handle'  => 'trackasiap-google-map-main',
									'src'   => TRACKASIAP_JS . 'maps.js',
									'deps'    => array(),
								);

								$scripts[] = array(
									'handle'  => 'trackasiap-trackasia-gl',
									'src'   => $trackasia_map_api,
									'deps'    => array('trackasiap-google-map-main'),
								);

								$scripts[] = array(
									'handle'  => 'trackasiap-backend',
									'src'   => TRACKASIAP_JS . 'minified/trackasia_backend.min.js',
									'deps'    => array('trackasiap-google-map-main'),
								);

								if ($scripts) {
									foreach ($scripts as $script) {
										wp_enqueue_script($script['handle'], $script['src'], $script['deps'], false);
									}
								}

								$trackasia_local = array();
								$trackasia_local['language'] = 'en';
								$trackasia_local['urlforajax'] = admin_url('admin-ajax.php');
								$trackasia_local['hide'] = esc_html__('Hide', 'wp-trackasia-maps');
								$trackasia_local['nonce']  = wp_create_nonce('fc-call-nonce');
								$trackasia_local['ajaxnonce'] = wp_create_nonce('fc_communication');
								$trackasia_local['text_editable']   = array('.fc-text', '.fc-post-link', '.place_title', '.fc-item-content', '.trackasia_locations_content');
								$trackasia_local['bg_editable']     = array('.fc-bg', '.fc-item-box', '.fc-pagination', '.trackasia_locations');
								$trackasia_local['margin_editable'] = array('.fc-margin', '.fc-item-title', '.trackasia_locations_head', '.fc-item-content', '.fc-item-meta');
								$trackasia_local['full_editable']   = array('.fc-css', '.fc-item-title', '.trackasia_locations_head', '.fc-readmore-link', '.fc-item-meta', 'a.page-numbers', '.current', '.trackasia_location_meta');
								$trackasia_local['confirm_location_delete'] = esc_html__('Do you really want to delete this location?', 'wp-trackasia-maps');
								$trackasia_local['confirm_map_delete'] = esc_html__('Do you really want to delete this map?', 'wp-trackasia-maps');
								$trackasia_local['confirm_category_delete'] = esc_html__('Do you really want to delete this category?', 'wp-trackasia-maps');
								$trackasia_local['confirm_route_delete'] = esc_html__('Do you really want to delete this route?', 'wp-trackasia-maps');
								$trackasia_local['confirm_record_delete'] = esc_html__('Do you really want to delete this record?', 'wp-trackasia-maps');
								$trackasia_local['no_record_for_bulk_delete'] = esc_html__('Please select some records first to apply bulk action on them.', 'wp-trackasia-maps');
								$trackasia_local['confirm_bulk_delete'] = esc_html__('Are you sure you want to delete the selected records ?', 'wp-trackasia-maps');
								$trackasia_local['confirm_overwrite_db'] = esc_html__('Overwrite existing google maps database?', 'wp-trackasia-maps');
								$trackasia_local['referrer_copied'] = esc_html__('Referrer Was Copied', 'wp-trackasia-maps');
								$trackasia_local['do_referrer_copy'] = esc_html__('Copy HTTP Referrer To Clipboard', 'wp-trackasia-maps');
								$trackasia_local['trackasia_assets'] = TRACKASIAP_JS;

								wp_localize_script('trackasiap-google-map-main', 'trackasia_local', $trackasia_local);
								wp_localize_script('trackasiap-backend', 'settings_obj', $trackasia_local);

								$trackasia_js_lang = array();
								$trackasia_js_lang['confirm'] = esc_html__('Are you sure to delete item?', 'wp-trackasia-maps');
								wp_localize_script('trackasiap-backend-google-maps', 'trackasia_js_lang', $trackasia_js_lang);
								$admin_styles = array(
									'font_awesome_minimised' => TRACKASIAP_CSS . 'font-awesome.min.css',
									'trackasiap-map-bootstrap' => TRACKASIAP_CSS . 'trackasia-ui.css',
									'trackasiap-backend-google-map' => TRACKASIAP_CSS . 'backend.css',
									'trackasiap-backend-bootstrap-modal' => TRACKASIAP_CSS . 'bootstrap-modal.css'
								);

								if ($admin_styles) {
									foreach ($admin_styles as $admin_style_key => $admin_style_value) {
										wp_enqueue_style($admin_style_key, $admin_style_value);
									}
								}
							}

							public static function trackasia_apply_placeholders($content)
							{

								$data['marker_id']                 = 1;
								$data['marker_title']              = 'New York, NY, United States';
								$data['marker_address']            = 'New York, NY, United States';
								$data['marker_message']            = 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam.';
								$data['marker_category']           = 'Real Estate';
								$data['marker_icon']               = TRACKASIAP_IMAGES . 'default_marker.png';
								$data['marker_latitude']           = '40.7127837';
								$data['marker_longitude']          = '-74.00594130000002';
								$data['marker_city']               = 'New York';
								$data['marker_state']              = 'NY';
								$data['marker_country']            = 'United States';
								$data['marker_zoom']               = '5';
								$data['marker_postal_code']        = '10002';
								$data['extra_field_slug']          = 'color';
								$data['marker_featured_image_src'] = TRACKASIAP_IMAGES . 'sample.jpg';
								$data['marker_image']              = '<img class="fc-item-featured_image  fc-item-large" src="' . TRACKASIAP_IMAGES . 'sample.jpg' . '" />';
								$data['marker_featured_image']     = '<img class="fc-item-featured_image  fc-item-large" src="' . TRACKASIAP_IMAGES . 'sample.jpg' . '" />';
								$data['post_title']                = 'Lorem ipsum dolor sit amet, consectetur';
								$data['post_link']                 = '#';
								$data['post_excerpt']              = 'Lorem ipsum dolor sit amet, consectetur';
								$data['post_content']              = 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat.';
								$data['post_categories']           = 'city tour';
								$data['post_tags']                 = 'WordPress, plugins, google maps';
								$data['post_featured_image']       = '<img class="fc-item-featured_image  fc-item-large" src="' . TRACKASIAP_IMAGES . 'sample.jpg' . '" />';
								$data['post_author']               = 'FlipperCode';
								$data['post_comments']             = '<i class="fci fci-comment"></i> 10';
								$data['view_count']                = '<i class="fci fci-heart"></i> 1';

								foreach ($data as $key => $value) {
									if (strstr($key, 'marker_featured_image_src') === false && strstr($key, 'marker_icon') === false && strstr($key, 'post_link') === false && strstr($key, 'marker_zoom') === false && strstr($key, 'marker_id') === false && strstr($key, 'post_title') === false) {
										$content = str_replace("{{$key}}", $value . '<span class="fc-hidden-placeholder">{' . $key . '}</span>', $content);
									} else {
										$content = str_replace("{{$key}}", $value, $content);
									}
								}

								return $content;
							}

							/**
							 * Eneque scripts at backend overview page only.
							 */
							function trackasia_overview_page_styles($hook)
							{
								if ($hook == 'plugins.php') {
									wp_enqueue_style('trackasiap-modal-css', plugin_dir_url(__FILE__) . 'assets/css/modal.css');
									if (function_exists('trackasia_add_feedback_form')) {
										trackasia_add_feedback_form();
									}
								}
							}

							/**
							 * Load plugin language file.
							 */
							function trackasia_load_plugin_languages()
							{

								$this->modules = apply_filters('trackasia_extensions', $this->modules);
								load_plugin_textdomain('wp-trackasia-maps', false, TRACKASIAP_FOLDER . '/lang');
							}
							/**
							 * Call hook on plugin activation for both multi-site and single-site.
							 */
							function trackasia_plugin_activation($network_wide)
							{

								if (is_multisite() && $network_wide) {
									global $wpdb;
									$currentblog = $wpdb->blogid;
									$activated = array();
									$sql = "SELECT blog_id FROM {$wpdb->blogs}";
									$blog_ids = $wpdb->get_col($wpdb->prepare($sql, null));

									foreach ($blog_ids as $blog_id) {
										switch_to_blog($blog_id);
										$this->trackasia_activation();
										$activated[] = $blog_id;
									}
									switch_to_blog($currentblog);
								} else {
									$this->trackasia_activation();
								}
							}
							/**
							 * Call hook on plugin deactivation for both multi-site and single-site.
							 */
							function trackasia_plugin_deactivation()
							{

								if (is_multisite() && $network_wide) {
									global $wpdb;
									$currentblog = $wpdb->blogid;
									$activated = array();
									$sql = "SELECT blog_id FROM {$wpdb->blogs}";
									$blog_ids = $wpdb->get_col($wpdb->prepare($sql, null));

									foreach ($blog_ids as $blog_id) {
										switch_to_blog($blog_id);
										$this->trackasia_deactivation();
										$activated[] = $blog_id;
									}

									switch_to_blog($currentblog);
								} else {
									$this->trackasia_deactivation();
								}
							}

							/**
							 * Perform tasks on new blog create and table install.
							 */

							function trackasia_on_blog_new_generate($blog_id, $user_id, $domain, $path, $site_id, $meta)
							{

								if (is_plugin_active_for_network(plugin_basename(__FILE__))) {
									switch_to_blog($blog_id);
									$this->trackasia_activation();
									restore_current_blog();
								}
							}

							/**
							 * Perform tasks on when blog deleted and remove plugin tables.
							 */

							function trackasia_on_blog_delete($tables)
							{

								global $wpdb;
								$tables[] = str_replace($wpdb->base_prefix, $wpdb->prefix, TBL_LOCATION);
								$tables[] = str_replace($wpdb->base_prefix, $wpdb->prefix, TBL_GROUPMAP);
								$tables[] = str_replace($wpdb->base_prefix, $wpdb->prefix, TBL_MAP);
								$tables[] = str_replace($wpdb->base_prefix, $wpdb->prefix, TBL_ROUTES);
								return $tables;
							}
							/**
							 * Create choose icon tab in media manager.
							 * @param  array $tabs Current Tabs.
							 * @return array       New Tabs.
							 */
							function trackasia_google_map_tabs_filter($tabs)
							{

								$newtab = array('ell_insert_gmap_tab' => esc_html__('Choose Icons', 'wp-trackasia-maps'));
								return array_merge($tabs, $newtab);
							}
							/**
							 * Intialize wp_iframe for icons tab
							 * @return [type] [description]
							 */
							function trackasia_google_map_media_upload_tab()
							{

								wp_enqueue_style('marker-listing-styling', TRACKASIAP_CSS . 'marker-category-style.css');
								wp_enqueue_script('marker-listing-script', TRACKASIAP_JS . 'marker-category-script.js');
								return wp_iframe(array($this, 'media_trackasia_google_map_icon'), array());
							}

							/**
							 * Read images/icons folder.
							 */
							function media_trackasia_google_map_icon()
							{

								wp_enqueue_style('media');
								media_upload_header();
								$form_action_url = site_url("wp-admin/media-upload.php?type={$GLOBALS['type']}&tab=ell_insert_gmap_tab", 'admin');
									?>

			<form enctype="multipart/form-data" method="post" action="<?php echo esc_attr($form_action_url); ?>" class="media-upload-form marker-choose-form" id="library-form">
				<h3 class="media-title"><?php esc_html_e('Choose icon', 'wp-trackasia-maps') ?></h3>
				<input name="trackasia_search_icon" id="trackasia_search_icon" type='text' value="" placeholder="<?php esc_html_e('Search icons', 'wp-trackasia-maps'); ?>" />
				<div class="select_icons_container">
					<ul id="select_icons">
						<?php
								$dir = TRACKASIAP_ICONS_DIR;
								$file_display = array('jpg', 'jpeg', 'png', 'gif');

								if (file_exists($dir) == false) {
									echo 'Directory \'', esc_html($dir), '\' not found!';
								} else {
									$dir_contents = scandir($dir);
									foreach ($dir_contents as $file) {
										$image_data = explode('.', $file);
										$file_type = strtolower(end($image_data));
										if ('.' !== $file && '..' !== $file && true == in_array($file_type, $file_display)) {
						?>
									<li class="read_icons">
										<img alt="<?php echo esc_attr($image_data[0]); ?>" title="<?php echo esc_attr($image_data[0]); ?>" src="<?php echo esc_url(TRACKASIAP_ICONS . $file); ?>" />
									</li>
						<?php
										}
									}
								}
								if (isset($_GET['target']))
									$target = sanitize_key($_GET['target']);
						?>
					</ul>
					<button type="button" class="button set_marker_cat_button" data-message="<?php esc_html_e('Please choose marker icon for the category.', 'wp-trackasia-maps'); ?>" data-target="<?php echo esc_attr($target); ?>" value="1" onclick="add_icon_to_images();" name="send[<?php echo esc_attr($picid) ?>]"><?php esc_html_e('Insert into Post', 'wp-trackasia-maps') ?>
					</button>
				</div>
			</form>
<?php
							}

							function trackasia_hide_buy_notice()
							{

								$nonce = $_REQUEST['ajaxnonce'];
								if (
									!isset($_REQUEST['ajaxnonce']) ||
									! wp_verify_nonce($nonce, 'fc_communication') ||
									!current_user_can('manage_options')
								) {

									$result = array('status' => '0');
								} else {
									$data = maybe_unserialize(get_option('trackasia_ignore_buy_pro'));
									$data['trackasia_hide_buy_notice'] = 'true';
									update_option('trackasia_ignore_buy_pro', serialize($data));
									$result = array('status' => '1');
								}

								echo json_encode($result);
								exit;
							}

							function trackasia_show_buynow_notice($pluginheader)
							{

								if (isset($_GET['page']) && !empty($_GET['page']) && strpos(sanitize_key($_GET['page']), 'trackasiap') !== false) {

									$ignore = maybe_unserialize(get_option('trackasia_ignore_buy_pro'));

									if (isset($ignore['trackasia_hide_buy_notice']) && !empty($ignore['trackasia_hide_buy_notice'])) {
									} else {

										$premium_plugin = '<a target="_blank" href="https://www.wpmapspro.com/?utm_source=wordpress&utm_medium=liteversion&utm_campaign=freemium&utm_id=freemium">' . esc_html__('Pro', 'wp-trackasia-maps') . '</a>';

										$pluginheader .= '<div class="trackasia-ui">
						<div class="fc-main"><div class="buy_premium notice notice-success is-dismissible">' . sprintf(esc_html__('Upgrade hassle-free to %s version & unlock listing, routes,  directions, filtration, drawing & more features.', 'wp-trackasia-maps'), $premium_plugin) . ' <a class="buy_now_link" href="https://www.wpmapspro.com/?utm_source=wordpress&utm_medium=liteversion&utm_campaign=freemium&utm_id=freemium" target="_blank">' . esc_html__('Upgrade Now', 'wp-trackasia-maps') . '</a> or <a onclick="return false;" class="fc_ignore_notice hide_buy_notice" href="#">' . esc_html('Don\'t show again', 'wp-trackasia-maps') . '</a></div></div></div>';
									}
								}

								return $pluginheader;
							}

							function trackasia_hide_sample_notice()
							{

								$nonce = $_REQUEST['ajaxnonce'];
								if (
									!isset($_REQUEST['ajaxnonce']) ||
									!wp_verify_nonce($nonce, 'fc_communication') ||
									!current_user_can('manage_options')
								) {

									$result = array('status' => '0');
								} else {
									$data = maybe_unserialize(get_option('trackasia_ignore_buy_pro'));
									$data['trackasia_hide_sample_notice'] = 'true';
									update_option('trackasia_ignore_buy_pro', serialize($data));
									$result = array('status' => '1');
								}

								echo json_encode($result);
								exit;
							}

							/**
							 * Perform tasks on plugin deactivation.
							 */
							function trackasia_deactivation() {}

							/**
							 * Perform tasks on plugin deactivation.
							 */
							function trackasia_activation()
							{

								global $wpdb;

								require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

								$modules = $this->modules;
								$pagehooks = array();
								if (is_array($modules)) {
									foreach ($modules as $module) {
										$object = new $module;
										if (method_exists($object, 'install')) {
											$tables[] = $object->install();
										}
									}
								}

								if (isset($tables) && !empty($tables) && is_array($tables)) {
									foreach ($tables as $i => $sql) {
										dbDelta($sql);
									}
								}

								$data = maybe_unserialize(get_option('trackasia_ignore_buy_pro'));
								if (!isset($data['trackasia_show_sample_notice_time']) || empty($data['trackasia_show_sample_notice_time'])) {
									$data['trackasia_show_sample_notice_time'] = date('Y-m-d', strtotime('+3days'));
									update_option('trackasia_ignore_buy_pro', serialize($data));
								}
							}

							/**
							 * Define all constants.
							 */
							private function trackasia_define_constants()
							{

								global $wpdb;

								if (! defined('TRACKASIAP_SLUG'))
									define('TRACKASIAP_SLUG', 'trackasia_how_overview');

								if (! defined('TRACKASIAP_VERSION'))
									define('TRACKASIAP_VERSION', '1.0.0');

								if (! defined('TRACKASIAP_FOLDER'))
									define('TRACKASIAP_FOLDER', basename(dirname(__FILE__)));

								if (! defined('TRACKASIAP_DIR'))
									define('TRACKASIAP_DIR', plugin_dir_path(__FILE__));

								if (! defined('TRACKASIAP_ICONS_DIR'))
									define('TRACKASIAP_ICONS_DIR', TRACKASIAP_DIR . '/assets/images/icons/');

								if (! defined('TRACKASIAP_CORE_CLASSES'))
									define('TRACKASIAP_CORE_CLASSES', TRACKASIAP_DIR . 'core/');

								if (! defined('TRACKASIAP_PLUGIN_CLASSES'))
									define('TRACKASIAP_PLUGIN_CLASSES', TRACKASIAP_DIR . 'classes/');

								if (! defined('TRACKASIAP_TEMPLATES'))
									define('TRACKASIAP_TEMPLATES', TRACKASIAP_DIR . 'templates/');

								if (! defined('TRACKASIAP_MODEL'))
									define('TRACKASIAP_MODEL', TRACKASIAP_DIR . 'modules/');

								if (! defined('TRACKASIAP_CONTROLLER'))
									define('TRACKASIAP_CONTROLLER', TRACKASIAP_CORE_CLASSES);

								if (! defined('TRACKASIAP_CORE_CONTROLLER_CLASS'))
									define('TRACKASIAP_CORE_CONTROLLER_CLASS', TRACKASIAP_CORE_CLASSES . 'class.controller.php');

								if (! defined('TRACKASIAP_MODEL'))
									define('TRACKASIAP_MODEL', TRACKASIAP_DIR . 'modules/');

								if (! defined('TRACKASIAP_URL'))
									define('TRACKASIAP_URL', plugin_dir_url(TRACKASIAP_FOLDER) . TRACKASIAP_FOLDER . '/');

								if (! defined('TRACKASIAP_CSS'))
									define('TRACKASIAP_CSS', TRACKASIAP_URL . 'assets/css/');

								if (! defined('TRACKASIAP_TEMPLATES_URL'))
									define('TRACKASIAP_TEMPLATES_URL', TRACKASIAP_URL . 'templates/');

								if (! defined('TRACKASIAP_JS'))
									define('TRACKASIAP_JS', TRACKASIAP_URL . 'assets/js/');

								if (! defined('TRACKASIAP_IMAGES'))
									define('TRACKASIAP_IMAGES', TRACKASIAP_URL . 'assets/images/');

								if (! defined('TRACKASIAP_ICONS'))
									define('TRACKASIAP_ICONS', TRACKASIAP_URL . 'assets/images/icons/');

								if (! defined('TBL_LOCATION'))
									define('TBL_LOCATION', $wpdb->prefix . 'map_locations');

								if (! defined('TBL_GROUPMAP'))
									define('TBL_GROUPMAP', $wpdb->prefix . 'group_map');

								if (! defined('TBL_MAP'))
									define('TBL_MAP', $wpdb->prefix . 'create_map');

								if (! defined('TBL_ROUTES'))
									define('TBL_ROUTES', $wpdb->prefix . 'map_routes');

								if (! defined('TRACKASIAP_PRO_IMAGES'))
									define('TRACKASIAP_PRO_IMAGES', TRACKASIAP_URL . 'assets/images/pro/');

								if (! defined('TRACKASIAP_PREMIUM_LINK'))
									define('TRACKASIAP_PREMIUM_LINK', '<a href="javascript:void(0);" class="get_pro">' . esc_html__('PRO', 'wp-trackasia-maps') . '</a>');
							}
							/**
							 * Load all required core classes.
							 */
							private function trackasia_load_files()
							{

								$coreInitialisationFile = plugin_dir_path(__FILE__) . 'core/class.initiate-core.php';
								if (file_exists($coreInitialisationFile)) {
									require_once($coreInitialisationFile);
								}

								$blockFile = TRACKASIAP_PLUGIN_CLASSES . 'class-wp-maps-block.php';
								if (file_exists($blockFile)) {
									require_once($blockFile);
								}

								//Load Plugin Files	
								$plugin_files_to_include = array(
									'trackasiap-check-cookies.php',
									'trackasiap-template.php',
									'trackasiap-controller.php',
									'trackasiap-model.php',
									'class.map-widget.php',
									'trackasiap-feedback-form.php'
								);
								foreach ($plugin_files_to_include as $file) {

									if (file_exists(TRACKASIAP_PLUGIN_CLASSES . $file))
										require_once(TRACKASIAP_PLUGIN_CLASSES . $file);
								}
								// Load all modules.
								$core_modules = array('overview', 'settings', 'map', 'location', 'group_map');
								if (is_array($core_modules)) {
									foreach ($core_modules as $module) {

										$file = TRACKASIAP_MODEL . $module . '/model.' . $module . '.php';

										if (file_exists($file)) {
											include_once($file);
											$class_name = 'TRACKASIAP_Model_' . ucwords($module);
											array_push($this->modules, $class_name);
										}
									}
								}
							}
						}
					}

					new FC_TrackAsia_Maps();
